﻿Option Explicit On
Option Compare Binary
Option Strict On
Imports System
Imports System.String
Imports System.Globalization
Imports System.Threading.Thread
Imports System.Windows.Forms

Public Class DataGridViewClassesColumn

#Region "TextBoxColumn"

    Public Class DataGridViewTextBoxColumn
        '
        'этот пользовательский тип столбца необходим для упрощения ввода текстовых данных
        'пример
        'для столбцов:
        '   "Column1",
        '   "Column2",
        '   "Column3" 
        '   элемента управления DataGridView - grdEdit
        '   и максимального количества строк текста, выведенного в элементе управления TextBox, равного 4 
        '   Private Sub Me_Load(ByVal sender As Object, ByVal e As EventArgs)
        '       ...
        '       Dim xxx As New DataGridViewClassesColumn.DataGridViewTextBoxColumn
        '       Call xxx.TextBoxColumn(grdEdit, {"Column1", "Column2", "Column3"})
        '       ...
        '   End Sub
        'количество столбцов неограничено
        '
        Private dgv As DataGridView
        Private tb As TextBox
        Private CollumnsNames() As String 'имена столбцов DataGridView

        Public Sub TextBoxColumn(ByVal dgvEnterKey As DataGridView, ByVal Names() As String)
            'dgvEnterKey - DataGridView
            'Names() - имена столбцов DataGridView
            If UBound(Names) < 0 Then
                bMessage(1, "Внимание", "Проверьте аргументы процедуры TextBoxColumn", False)
                Application.Exit()
            End If
            ReDim CollumnsNames(UBound(Names))
            For i As Integer = 0 To UBound(Names)
                CollumnsNames(i) = Names(i)
            Next
            dgv = CType(dgvEnterKey, DataGridView)
            tb = New TextBox
            With tb
                .Visible = False
                .BorderStyle = BorderStyle.None
                .ScrollBars = ScrollBars.Vertical
                .TextAlign = HorizontalAlignment.Left
                .AutoSize = False
                .WordWrap = True
                .Multiline = True
            End With
            With dgv
                .Controls.Add(tb)
                AddHandler .CellEnter, AddressOf dgv_CellEnter
            End With
        End Sub

        Private Sub dgv_CellEnter(ByVal sender As Object, ByVal e As DataGridViewCellEventArgs)
            For i As Integer = 0 To UBound(CollumnsNames)
                If e.ColumnIndex = dgv.Columns(CollumnsNames(i)).Index Then
                    With dgv.GetCellDisplayRectangle(e.ColumnIndex, e.RowIndex, True)
                        tb.Bounds = New Rectangle(.X, .Y, .Width - 1, .Height - 1)
                    End With
                    With dgv.DefaultCellStyle
                        tb.BackColor = .BackColor
                        tb.Font = .Font
                        tb.ForeColor = .ForeColor
                    End With
                    With tb
                        Try
                            .Text = dgv.CurrentCell.Value.ToString
                            .SelectionStart = .Text.Length
                        Catch ex As Exception
                            .Text = Empty
                        End Try
                        .Visible = True
                        .Focus()
                        AddHandler .KeyPress, AddressOf tb_KeyPress
                    End With
                    AddHandler dgv.CellLeave, AddressOf dgv_CellLeave
                    Exit Sub
                End If
            Next
        End Sub

        Private Sub dgv_CellLeave(ByVal sender As Object, ByVal e As DataGridViewCellEventArgs)
            With dgv
                .CurrentCell.Value = tb.Text
                tb.Visible = False
                .Focus()
                .AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.AllCells
                RemoveHandler .CellLeave, AddressOf dgv_CellLeave
            End With
            RemoveHandler tb.KeyPress, AddressOf tb_KeyPress
        End Sub

        Private Sub tb_KeyPress(ByVal sender As Object, ByVal e As KeyPressEventArgs)
            tb.Focus()
        End Sub

    End Class

#End Region

#Region "MaskedTextBoxColumn"

    Public Class DataGridViewMaskedTextBoxColumn
        '
        'этот пользовательский тип столбца необходим для предотвращения ошибок ввода
        'пример
        'для столбцов:
        '   "Column1" (маска "####"),
        '   "Column2" (маска "#########"),
        '   "Column3" (маска "####")
        '   элемента управления DataGridView grdEdit
        '   Private Sub Me_Load(ByVal sender As Object, ByVal e As EventArgs)
        '       ...
        '       Dim xxx As New DataGridViewClassesColumn.DataGridViewMaskedTextBoxColumn
        '       Call xxx.MaskedTextBoxColumn(grdEdit, {"Column1", "Column2", "Column3"}, {"####", "#########", "####"})
        '       ...
        '   End Sub
        'количество столбцов неограничено
        'теперь в этих столбцах пользователь может вводить только числа
        '
        Private dgv As DataGridView
        Private mtb As MaskedTextBox
        Private CollumnsNames() As String 'имена столбцов DataGridView
        Private CollumnsMasks() As String 'маски, соответствующие именам столбцов DataGridView

        Public Sub MaskedTextBoxColumn(ByVal dgvEnterKey As DataGridView, ByVal Names() As String, ByVal Masks() As String)
            'dgvEnterKey - DataGridView
            'Names() - имена столбцов DataGridView
            'Masks() - маски, соответствующие именам столбцов, DataGridView
            If UBound(Names) >= 0 And UBound(Names) = UBound(Masks) Then
                ReDim CollumnsNames(UBound(Names))
                ReDim CollumnsMasks(UBound(Masks))
                For i As Integer = 0 To UBound(Names)
                    CollumnsNames(i) = Names(i)
                    CollumnsMasks(i) = Masks(i)
                Next
            Else
                bMessage(1, "Внимание", "Проверьте аргументы процедуры MaskedTextBoxColumn", False)
                Application.Exit()
            End If
            dgv = CType(dgvEnterKey, DataGridView)
            mtb = New MaskedTextBox
            With mtb
                .Visible = False
                .AutoSize = False
                .BorderStyle = BorderStyle.None
            End With
            With dgv
                .Controls.Add(mtb)
                AddHandler .CellEnter, AddressOf dgv_CellEnter
            End With
        End Sub

        Private Sub dgv_CellEnter(ByVal sender As Object, ByVal e As DataGridViewCellEventArgs)
            For i As Integer = 0 To UBound(CollumnsNames)
                If e.ColumnIndex = dgv.Columns(CollumnsNames(i)).Index Then
                    With dgv.GetCellDisplayRectangle(e.ColumnIndex, e.RowIndex, True)
                        mtb.Bounds = New Rectangle(.X, .Y, .Width - 1, .Height - 1)
                    End With
                    With dgv.DefaultCellStyle
                        mtb.BackColor = .BackColor
                        mtb.Font = .Font
                        mtb.ForeColor = .ForeColor
                    End With
                    With mtb
                        .Mask = CollumnsMasks(i)
                        Try
                            .Text = dgv.CurrentCell.Value.ToString
                        Catch ex As Exception
                            .Text = Empty
                        End Try
                        .SelectionStart = .Text.Length
                        .Visible = True
                        .Focus()
                    End With
                    AddHandler dgv.CellLeave, AddressOf dgv_CellLeave
                    Exit Sub
                End If
            Next
        End Sub

        Private Sub dgv_CellLeave(ByVal sender As Object, ByVal e As DataGridViewCellEventArgs)
            With dgv
                .CurrentCell.Value = mtb.Text
                mtb.Visible = False
                .Focus()
                RemoveHandler .CellLeave, AddressOf dgv_CellLeave
            End With
        End Sub

    End Class

#End Region

#Region "CurrencyColumn"

    Public Class DataGridViewCurrencyColumn
        '
        'этот пользовательский тип столбца необходим для упрощения ввода денежного типа данных
        'пример
        'для столбцов:
        '   "Column1" (евро),
        '   "Column2" (рубль),
        '   "Column3" (доллар)
        '   элемента управления DataGridView grdEdit
        '   Private Sub Me_Load(ByVal sender As Object, ByVal e As EventArgs)
        '       ...
        '       Dim xxx As New DataGridViewClassesColumn.DataGridViewCurrencyColumn
        '       Call xxx.CurrencyColumn(grdEdit, {"Column1", "Column2", "Column3"}, {"fr-BE", "Ru-Ru", "en-US"})
        '       ...
        '   End Sub
        'Здсь "fr-BE" - евро, "Ru-Ru" - рубль, "en-US" - доллар
        'количество столбцов неограничено
        '
        Private dgv As DataGridView
        Private tb As TextBox
        Private CollumnsNames() As String 'имена столбцов DataGridView
        Private ArrayInformations() As String 'значения = "fr-BE", "Ru-Ru", "en-US", ..

        Public Sub CurrencyColumn(ByVal dgvEnterKey As DataGridView, ByVal Names() As String, ByVal ArrayCultureInformations() As String)
            'dgvEnterKey - DataGridView
            'Names() - имена столбцов DataGridView
            'CultureInformations(), значения = "fr-BE", "Ru-Ru", "en-US", ..
            If UBound(Names) >= 0 And UBound(Names) = UBound(ArrayCultureInformations) Then
                ReDim CollumnsNames(UBound(Names))
                ReDim ArrayInformations(UBound(ArrayCultureInformations))
                For i As Integer = 0 To UBound(Names)
                    CollumnsNames(i) = Names(i)
                    ArrayInformations(i) = ArrayCultureInformations(i)
                Next
            Else
                bMessage(1, "Внимание", "Проверьте аргументы процедуры CurrencyColumn", False)
                Application.Exit()
            End If
            dgv = CType(dgvEnterKey, DataGridView)
            tb = New TextBox
            With tb
                .BorderStyle = BorderStyle.None
                .AutoSize = False
                .Visible = False
                .ReadOnly = False
            End With
            With dgv
                .Controls.Add(tb)
                AddHandler .CellEnter, AddressOf dgv_CellEnter
            End With
        End Sub

        Private Sub dgv_CellEnter(ByVal sender As Object, ByVal e As DataGridViewCellEventArgs)
            For i As Integer = 0 To UBound(CollumnsNames)
                If e.ColumnIndex = dgv.Columns(CollumnsNames(i)).Index Then
                    CurrentThread.CurrentCulture = New CultureInfo(ArrayInformations(i))
                    With dgv.GetCellDisplayRectangle(e.ColumnIndex, e.RowIndex, True)
                        tb.Bounds = New Rectangle(.X, .Y, .Width - 1, .Height - 1)
                    End With
                    With dgv.DefaultCellStyle
                        tb.BackColor = .BackColor
                        tb.Font = .Font
                        tb.ForeColor = .ForeColor
                    End With
                    With tb
                        Try
                            .Text = CDec(dgv.CurrentCell.Value).ToString("C2")
                        Catch ex As Exception
                            .Text = 0.ToString("C2")
                        End Try
                        Dim parts() As String = .Text.Split(CChar("р"))
                        .Text = parts(0)
                        .Visible = True
                        .Focus()
                        AddHandler .KeyPress, AddressOf tb_KeyPress
                    End With
                    AddHandler dgv.CellLeave, AddressOf dgv_CellLeave
                    Exit Sub
                End If
            Next
        End Sub

        Private Sub tb_KeyPress(ByVal sender As Object, ByVal e As KeyPressEventArgs)
            With tb
                Dim p As String = "-.,0123456789"
                Dim c As Char = e.KeyChar
                Dim s As String = .Text & CStr(c)
                If p.IndexOf(c) >= 0 Or Char.IsPunctuation(c) Then
                    Try
                        If s <> "-" Then
                            If CDec(s) = 0 Then
                                .Text = Empty
                            End If
                        End If
                        If s.IndexOf(",,") >= 0 Or s.IndexOf("..") >= 0 Or s.IndexOf(".,") >= 0 Or s.IndexOf(",.") >= 0 Then
                            e.Handled = True
                        End If
                    Catch ex As Exception
                        e.Handled = True
                    End Try
                Else
                    If Char.IsLetter(c) Or c = CChar("~") Or c = CChar("`") Or c = CChar("=") Or c = CChar("|") Or c = CChar("'") Or c = CChar("^") Or c = CChar("+") Then e.Handled = True
                End If
                .Focus()
            End With
        End Sub

        Private Sub dgv_CellLeave(ByVal sender As Object, ByVal e As DataGridViewCellEventArgs)
            With dgv
                With .CurrentCell
                    Try
                        .Value = CDec(tb.Text).ToString("C2")
                    Catch ex As Exception
                        .Value = CDec(0).ToString("C2")
                    End Try
                End With
                CurrentThread.CurrentCulture = CurrentThread.CurrentCulture
                tb.Visible = False
                .Focus()
                RemoveHandler .CellLeave, AddressOf dgv_CellLeave
            End With
            RemoveHandler tb.KeyPress, AddressOf tb_KeyPress
        End Sub

    End Class

#End Region

#Region "ComboBoxColumn"

    Public Class DataGridViewComboBoxColumn
        '
        'этот пользовательский тип столбца необходим для упрощения ввода из поля со списком (ComboBox)
        'пример
        'для столбцов:
        '   "Column1" (поле со списком ComboBox1),
        '   "Column2" (поле со списком ComboBox2),
        '   "Column3" (поле со списком ComboBox3)
        '   элемента управления DataGridView grdEdit
        '   Private Sub Me_Load(ByVal sender As Object, ByVal e As EventArgs)
        '       ...
        '       Dim xxx As New DataGridViewClassesColumn.DataGridViewComboBoxColumn
        '       Call xxx.ComboBoxColumn(grdEdit, {"Column1", "Column2", "Column3"}, {ComboBox1, ComboBox2, ComboBox3})
        '       ...
        '   End Sub
        'количество столбцов неограничено
        '
        Private dgv As DataGridView
        Private cbo As New ComboBox
        Private CollumnsNames() As String 'имена столбцов DataGridView
        Private CollumnsComboBoxs() As ComboBox 'поля со списком (ComboBox), соответствующие именам столбцов DataGridView

        Public Sub ComboBoxColumn(ByVal dgvEnterKey As DataGridView, ByVal Names() As String, ByVal ComboBoxs() As ComboBox)
            'dgvEnterKey - DataGridView
            'Names() - имена столбцов DataGridView
            'ComboBoxs() - поля со списком (ComboBox), соответствующие именам столбцов DataGridView
            If UBound(Names) >= 0 And UBound(Names) = UBound(ComboBoxs) Then
                ReDim CollumnsNames(UBound(Names))
                ReDim CollumnsComboBoxs(UBound(ComboBoxs))
                For i As Integer = 0 To UBound(Names)
                    CollumnsNames(i) = Names(i)
                    CollumnsComboBoxs(i) = ComboBoxs(i)
                Next
            Else
                bMessage(1, "Внимание", "Проверьте аргументы процедуры ComboBoxColumn", False)
                Application.Exit()
            End If
            dgv = CType(dgvEnterKey, DataGridView)
            With cbo
                .Visible = False
                .FlatStyle = FlatStyle.Popup
                .DropDownStyle = ComboBoxStyle.DropDownList
            End With
            With dgv
                .Controls.Add(cbo)
                AddHandler .CellEnter, AddressOf dgv_CellEnter
            End With
        End Sub

        Private Sub dgv_CellEnter(ByVal sender As Object, ByVal e As DataGridViewCellEventArgs)
            For i As Integer = 0 To UBound(CollumnsNames)
                If e.ColumnIndex = dgv.Columns(CollumnsNames(i)).Index Then
                    With dgv.GetCellDisplayRectangle(e.ColumnIndex, e.RowIndex, True)
                        cbo.Bounds = New Rectangle(.X, .Y, .Width - 1, .Height - 1)
                    End With
                    With dgv.DefaultCellStyle
                        cbo.BackColor = .BackColor
                        cbo.Font = .Font
                        cbo.ForeColor = .ForeColor
                    End With
                    With cbo
                        For j As Integer = 0 To CollumnsComboBoxs(i).Items.Count - 1
                            .Items.Add(CollumnsComboBoxs(i).Items(j))
                        Next
                        .Text = dgv.CurrentCell.Value.ToString
                        .Visible = True
                        .Focus()
                    End With
                    AddHandler dgv.CellLeave, AddressOf dgv_CellLeave
                    AddHandler cbo.Leave, AddressOf cbo_Leave
                    Exit Sub
                End If
            Next
        End Sub

        Private Sub cbo_Leave(sender As Object, e As EventArgs)
            cbo.Focus()
        End Sub

        Private Sub dgv_CellLeave(ByVal sender As Object, ByVal e As DataGridViewCellEventArgs)
            With dgv
                .CurrentCell.Value = cbo.Text
                With cbo
                    .Visible = False
                    RemoveHandler .Leave, AddressOf cbo_Leave
                End With
                .Focus()
                RemoveHandler .CellLeave, AddressOf dgv_CellLeave
            End With
        End Sub

    End Class

#End Region

#Region "TemplateColumn"

    Public Class DataGridViewTemplateColumn
        '
        'этот пользовательский тип столбца необходим для фильтрации ввода с клавиатуры по шаблону
        'пример
        'для столбцов:
        '   "Column1" (шаблон "Template1"),
        '   "Column2" (шаблон "Template2"),
        '   "Column3" (шаблон "Template3") 
        '   элемента управления DataGridView - grdEdit
        '   и максимального количества строк текста, выведенного в элементе управления TextBox, равного 4 
        '   Private Sub Me_Load(ByVal sender As Object, ByVal e As EventArgs)
        '       ...
        '       Dim xxx As New DataGridViewClassesColumn.DataGridViewTemplateColumn
        '       Call xxx.TemplateColumn(grdEdit, {"Column1", "Column2", "Column3"}, {"Template1", "Template2", "Template3"})
        '       ...
        '   End Sub
        'количество столбцов неограничено
        '
        Private dgv As DataGridView
        Private tb As TextBox
        Private CollumnsNames() As String 'имена столбцов DataGridView
        Private CollumnsTemplates() As String 'шаблоны, соответствующие именам столбцов DataGridView
        Private pattern As String 'шаблон

        Public Sub TemplateColumn(ByVal dgvEnterKey As DataGridView, ByVal Names() As String, ByVal Templates() As String)
            'dgvEnterKey - DataGridView
            'Names() - имена столбцов DataGridView
            'Templates() - шаблоны, соответствующие именам столбцов DataGridView
            If UBound(Names) >= 0 And UBound(Names) = UBound(Templates) Then
                ReDim CollumnsNames(UBound(Names))
                ReDim CollumnsTemplates(UBound(Templates))
                For i As Integer = 0 To UBound(Names)
                    CollumnsNames(i) = Names(i)
                    CollumnsTemplates(i) = Templates(i)
                Next
            Else
                bMessage(1, "Внимание", "Проверьте аргументы процедуры TemplateColumn", False)
                Application.Exit()
            End If
            dgv = CType(dgvEnterKey, DataGridView)
            tb = New TextBox
            With tb
                .Visible = False
                .BorderStyle = BorderStyle.None
                .ScrollBars = ScrollBars.Vertical
                .TextAlign = HorizontalAlignment.Left
                .AutoSize = False
                .WordWrap = True
                .Multiline = True
            End With
            With dgv
                .Controls.Add(tb)
                AddHandler .CellEnter, AddressOf dgv_CellEnter
            End With
        End Sub

        Private Sub dgv_CellEnter(ByVal sender As Object, ByVal e As DataGridViewCellEventArgs)
            For i As Integer = 0 To UBound(CollumnsNames)
                If e.ColumnIndex = dgv.Columns(CollumnsNames(i)).Index Then
                    With dgv.GetCellDisplayRectangle(e.ColumnIndex, e.RowIndex, True)
                        tb.Bounds = New Rectangle(.X, .Y, .Width - 1, .Height - 1)
                    End With
                    With dgv.DefaultCellStyle
                        tb.BackColor = .BackColor
                        tb.Font = .Font
                        tb.ForeColor = .ForeColor
                    End With
                    pattern = CollumnsTemplates(i)
                    With tb
                        Try
                            .Text = dgv.CurrentCell.Value.ToString
                            .SelectionStart = .Text.Length
                        Catch ex As Exception
                            .Text = Empty
                        End Try
                        .Visible = True
                        .Focus()
                        AddHandler .KeyPress, AddressOf tb_KeyPress
                    End With
                    AddHandler dgv.CellLeave, AddressOf dgv_CellLeave
                    Exit Sub
                End If
            Next
        End Sub

        Private Sub dgv_CellLeave(ByVal sender As Object, ByVal e As DataGridViewCellEventArgs)
            With dgv
                .CurrentCell.Value = tb.Text
                tb.Visible = False
                .Focus()
                .AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.AllCells
                RemoveHandler .CellLeave, AddressOf dgv_CellLeave
            End With
            RemoveHandler tb.KeyPress, AddressOf tb_KeyPress
        End Sub

        Private Sub tb_KeyPress(ByVal sender As Object, ByVal e As KeyPressEventArgs)
            With e
                If Not (Char.IsControl(.KeyChar)) Then
                    If pattern.IndexOf(.KeyChar.ToString()) < 0 Then
                        .Handled = True
                    End If
                End If
            End With
            tb.Focus()
        End Sub

    End Class

#End Region

End Class
